@@ -1,4 +1,4 @@
-from aenum import Enum, IntEnum, UniqueEnum, AutoEnum, NamedTuple, TupleSize, AutoValue, AutoNumber, NoAlias, Unique, MultiValue
+from aenum import Enum, IntEnum, Flag, UniqueEnum, AutoEnum, NamedTuple, TupleSize, AutoValue, AutoNumber, NoAlias, Unique, MultiValue
 from aenum import AutoNumberEnum, OrderedEnum, unique, skip, extend_enum
 
 from collections import OrderedDict
@@ -6,6 +6,8 @@ from datetime import timedelta
 from pickle import dumps, loads, PicklingError, HIGHEST_PROTOCOL
 from unittest import TestCase, main
 
+import tempfile
+import textwrap
 import sys
 pyver = float('%s.%s' % sys.version_info[:2])
 
@@ -103,6 +105,15 @@ class TestEnumV3(TestCase):
         self.assertEqual(list(Color), [Color.red, Color.green, Color.blue])
         self.assertEqual(Color.red.value, 1)
 
+    def test_ignore_not_overridden(self):
+        with self.assertRaisesRegex(TypeError, 'object is not callable'):
+            class Color(Flag):
+                _ignore_ = 'irrelevent'
+                _settings_ = AutoValue
+                @property
+                def shade(self):
+                    print('I am light', self.name.lower())
+
     def test_magic_start(self):
         class Color(Enum, start=0):
             red, green, blue
@@ -469,6 +480,18 @@ class TestEnumV3(TestCase):
         self.assertEqual(len(Color), 4)
         self.assertEqual(Color.red.value, 1)
 
+    def test_extend_enum_generate(self):
+        class Foo(AutoEnum):
+            def _generate_next_value_(name, start, count, values, *args, **kwds):
+                return name
+            a
+            b
+        #
+        extend_enum(Foo, 'c')
+        self.assertEqual(Foo.a.value, 'a')
+        self.assertEqual(Foo.b.value, 'b')
+        self.assertEqual(Foo.c.value, 'c')
+
     def test_extend_enum_unique_with_duplicate(self):
         with self.assertRaises(ValueError):
             class Color(Enum, settings=Unique):
@@ -582,6 +605,210 @@ class TestEnumV3(TestCase):
             third
         self.assertEqual([Dupes.first, Dupes.second, Dupes.third], list(Dupes))
 
+    def test_order_as_function(self):
+        # first with _init_
+        class TestSequence(Enum):
+            _init_ = 'value, sequence'
+            _order_ = lambda member: member.sequence
+            item_id                  = 'An$(1,6)',      0     # Item Code
+            company_id               = 'An$(7,2)',      1     # Company Code
+            warehouse_no             = 'An$(9,4)',      2     # Warehouse Number
+            company                  = 'Hn$(13,6)',     3     # 4 SPACES + COMPANY
+            key_type                 = 'Cn$(19,3)',     4     # Key Type = '1**'
+            available                = 'Zn$(1,1)',      5     # Available?
+            contract_item            = 'Bn(2,1)',       6     # Contract Item?
+            sales_category           = 'Fn',            7     # Sales Category
+            gl_category              = 'Rn$(5,1)',      8     # G/L Category
+            warehouse_category       = 'Sn$(6,1)',      9     # Warehouse Category
+            inv_units                = 'Qn$(7,2)',     10     # Inv Units
+        for i, member in enumerate(TestSequence):
+            self.assertEqual(i, member.sequence)
+        ts = TestSequence
+        self.assertEqual(ts.item_id.name, 'item_id')
+        self.assertEqual(ts.item_id.value, 'An$(1,6)')
+        self.assertEqual(ts.item_id.sequence, 0)
+        self.assertEqual(ts.company_id.name, 'company_id')
+        self.assertEqual(ts.company_id.value, 'An$(7,2)')
+        self.assertEqual(ts.company_id.sequence, 1)
+        self.assertEqual(ts.warehouse_no.name, 'warehouse_no')
+        self.assertEqual(ts.warehouse_no.value, 'An$(9,4)')
+        self.assertEqual(ts.warehouse_no.sequence, 2)
+        self.assertEqual(ts.company.name, 'company')
+        self.assertEqual(ts.company.value, 'Hn$(13,6)')
+        self.assertEqual(ts.company.sequence, 3)
+        self.assertEqual(ts.key_type.name, 'key_type')
+        self.assertEqual(ts.key_type.value, 'Cn$(19,3)')
+        self.assertEqual(ts.key_type.sequence, 4)
+        self.assertEqual(ts.available.name, 'available')
+        self.assertEqual(ts.available.value, 'Zn$(1,1)')
+        self.assertEqual(ts.available.sequence, 5)
+        self.assertEqual(ts.contract_item.name, 'contract_item')
+        self.assertEqual(ts.contract_item.value, 'Bn(2,1)')
+        self.assertEqual(ts.contract_item.sequence, 6)
+        self.assertEqual(ts.sales_category.name, 'sales_category')
+        self.assertEqual(ts.sales_category.value, 'Fn')
+        self.assertEqual(ts.sales_category.sequence, 7)
+        self.assertEqual(ts.gl_category.name, 'gl_category')
+        self.assertEqual(ts.gl_category.value, 'Rn$(5,1)')
+        self.assertEqual(ts.gl_category.sequence, 8)
+        self.assertEqual(ts.warehouse_category.name, 'warehouse_category')
+        self.assertEqual(ts.warehouse_category.value, 'Sn$(6,1)')
+        self.assertEqual(ts.warehouse_category.sequence, 9)
+        self.assertEqual(ts.inv_units.name, 'inv_units')
+        self.assertEqual(ts.inv_units.value, 'Qn$(7,2)')
+        self.assertEqual(ts.inv_units.sequence, 10)
+        # and then without
+        class TestSequence(Enum):
+            _order_ = lambda member: member.value[1]
+            item_id                  = 'An$(1,6)',      0     # Item Code
+            company_id               = 'An$(7,2)',      1     # Company Code
+            warehouse_no             = 'An$(9,4)',      2     # Warehouse Number
+            company                  = 'Hn$(13,6)',     3     # 4 SPACES + COMPANY
+            key_type                 = 'Cn$(19,3)',     4     # Key Type = '1**'
+            available                = 'Zn$(1,1)',      5     # Available?
+            contract_item            = 'Bn(2,1)',       6     # Contract Item?
+            sales_category           = 'Fn',            7     # Sales Category
+            gl_category              = 'Rn$(5,1)',      8     # G/L Category
+            warehouse_category       = 'Sn$(6,1)',      9     # Warehouse Category
+            inv_units                = 'Qn$(7,2)',     10     # Inv Units
+        for i, member in enumerate(TestSequence):
+            self.assertEqual(i, member.value[1])
+        ts = TestSequence
+        self.assertEqual(ts.item_id.name, 'item_id')
+        self.assertEqual(ts.item_id.value, ('An$(1,6)', 0))
+        self.assertEqual(ts.company_id.name, 'company_id')
+        self.assertEqual(ts.company_id.value, ('An$(7,2)', 1))
+        self.assertEqual(ts.warehouse_no.name, 'warehouse_no')
+        self.assertEqual(ts.warehouse_no.value, ('An$(9,4)', 2))
+        self.assertEqual(ts.company.name, 'company')
+        self.assertEqual(ts.company.value, ('Hn$(13,6)', 3))
+        self.assertEqual(ts.key_type.name, 'key_type')
+        self.assertEqual(ts.key_type.value, ('Cn$(19,3)', 4))
+        self.assertEqual(ts.available.name, 'available')
+        self.assertEqual(ts.available.value, ('Zn$(1,1)', 5))
+        self.assertEqual(ts.contract_item.name, 'contract_item')
+        self.assertEqual(ts.contract_item.value, ('Bn(2,1)', 6))
+        self.assertEqual(ts.sales_category.name, 'sales_category')
+        self.assertEqual(ts.sales_category.value, ('Fn', 7))
+        self.assertEqual(ts.gl_category.name, 'gl_category')
+        self.assertEqual(ts.gl_category.value, ('Rn$(5,1)', 8))
+        self.assertEqual(ts.warehouse_category.name, 'warehouse_category')
+        self.assertEqual(ts.warehouse_category.value, ('Sn$(6,1)', 9))
+        self.assertEqual(ts.inv_units.name, 'inv_units')
+        self.assertEqual(ts.inv_units.value, ('Qn$(7,2)', 10))
+        # then with _init_ but without value
+        with self.assertRaises(TypeError):
+            class TestSequence(Enum):
+                _init_ = 'sequence'
+                _order_ = lambda member: member.sequence
+                item_id                  = 'An$(1,6)',      0     # Item Code
+                company_id               = 'An$(7,2)',      1     # Company Code
+                warehouse_no             = 'An$(9,4)',      2     # Warehouse Number
+                company                  = 'Hn$(13,6)',     3     # 4 SPACES + COMPANY
+                key_type                 = 'Cn$(19,3)',     4     # Key Type = '1**'
+                available                = 'Zn$(1,1)',      5     # Available?
+                contract_item            = 'Bn(2,1)',       6     # Contract Item?
+                sales_category           = 'Fn',            7     # Sales Category
+                gl_category              = 'Rn$(5,1)',      8     # G/L Category
+                warehouse_category       = 'Sn$(6,1)',      9     # Warehouse Category
+                inv_units                = 'Qn$(7,2)',     10     # Inv Units
+        # finally, out of order so Python 3 barfs
+        with self.assertRaises(TypeError):
+            class TestSequence(Enum):
+                _init_ = 'sequence'
+                _order_ = lambda member: member.sequence
+                item_id                  = 'An$(1,6)',      0     # Item Code
+                warehouse_no             = 'An$(9,4)',      2     # Warehouse Number
+                company                  = 'Hn$(13,6)',     3     # 4 SPACES + COMPANY
+                company_id               = 'An$(7,2)',      1     # Company Code
+                inv_units                = 'Qn$(7,2)',     10     # Inv Units
+                available                = 'Zn$(1,1)',      5     # Available?
+                contract_item            = 'Bn(2,1)',       6     # Contract Item?
+                sales_category           = 'Fn',            7     # Sales Category
+                key_type                 = 'Cn$(19,3)',     4     # Key Type = '1**'
+                gl_category              = 'Rn$(5,1)',      8     # G/L Category
+                warehouse_category       = 'Sn$(6,1)',      9     # Warehouse Category
+
+    if pyver >= 3.3:
+        def test_missing(self):
+            class Color(Enum):
+                red = 1
+                green = 2
+                blue = 3
+                @classmethod
+                def _missing_(cls, item):
+                    if item == 'three':
+                        return cls.blue
+                    elif item == 'bad return':
+                        # trigger internal error
+                        return 5
+                    elif item == 'error out':
+                        raise ZeroDivisionError
+                    else:
+                        # trigger not found
+                        return None
+            self.assertIs(Color('three'), Color.blue)
+            self.assertRaises(ValueError, Color, 7)
+            try:
+                Color('bad return')
+            except TypeError as exc:
+                self.assertTrue(isinstance(exc.__context__, ValueError))
+            else:
+                raise Exception('Exception not raised.')
+            try:
+                Color('error out')
+            except ZeroDivisionError as exc:
+                self.assertTrue(isinstance(exc.__context__, ValueError))
+            else:
+                raise Exception('Exception not raised.')
+
+    def test_enum_of_types(self):
+        """Support using Enum to refer to types deliberately."""
+        class MyTypes(Enum):
+            i = int
+            f = float
+            s = str
+        self.assertEqual(MyTypes.i.value, int)
+        self.assertEqual(MyTypes.f.value, float)
+        self.assertEqual(MyTypes.s.value, str)
+        class Foo:
+            pass
+        class Bar:
+            pass
+        class MyTypes2(Enum):
+            a = Foo
+            b = Bar
+        self.assertEqual(MyTypes2.a.value, Foo)
+        self.assertEqual(MyTypes2.b.value, Bar)
+        class SpamEnumNotInner:
+            pass
+        class SpamEnum(Enum):
+            spam = SpamEnumNotInner
+        self.assertEqual(SpamEnum.spam.value, SpamEnumNotInner)
+
+    def test_nested_classes_in_enum_do_not_create_members(self):
+        """Support locally-defined nested classes."""
+        # manually set __qualname__ to remove testing framework noise
+        class Outer(Enum):
+            __qualname__ = "Outer"
+            a = 1
+            b = 2
+            class Inner(Enum):
+                __qualname__ = "Outer.Inner"
+                foo = 10
+                bar = 11
+        self.assertTrue(isinstance(Outer.Inner, type))
+        self.assertEqual(Outer.a.value, 1)
+        self.assertEqual(Outer.Inner.foo.value, 10)
+        self.assertEqual(
+            list(Outer.Inner),
+            [Outer.Inner.foo, Outer.Inner.bar],
+            )
+        self.assertEqual(
+            list(Outer),
+            [Outer.a, Outer.b],
+            )
+
     if pyver == 3.4:
         def test_class_nested_enum_and_pickle_protocol_four(self):
             # would normally just have this directly in the class namespace
@@ -610,6 +837,20 @@ class TestEnumV3(TestCase):
                     protocol=(0, HIGHEST_PROTOCOL))
 
     if pyver >= 3.4:
+        def test_enum_injection(self):
+            class Color(Enum):
+                _order_ = 'BLACK WHITE'
+                BLACK = Color('black', '#000')
+                WHITE = Color('white', '#fff')
+
+                def __init__(self, label, hex):
+                    self.label = label
+                    self.hex = hex
+
+            self.assertEqual([Color.BLACK, Color.WHITE], list(Color))
+            self.assertEqual(Color.WHITE.hex, '#fff')
+            self.assertEqual(Color.BLACK.label, 'black')
+
         def test_subclasses_with_getnewargs_ex(self):
             class NamedInt(int):
                 __qualname__ = 'NamedInt'       # needed for pickle protocol 4
@@ -785,5 +1026,124 @@ class TestNamedTupleV3(TestCase):
         self.assertRaises(TypeError, Book, author='Steven Brust')
 
 
+
+class TestStackoverflowAnswersV3(TestCase):
+
+    def test_self_referential_directions(self):
+        # https://stackoverflow.com/a/64000706/208880
+        class Directions(Enum):
+            #
+            NORTH = 1, 0
+            WEST = 0, 1
+            SOUTH = -1, 0
+            EAST = 0, -1
+            #
+            def __init__(self, x, y):
+                self.x = x
+                self.y = y
+                if len(self.__class__):
+                    # make links
+                    all = list(self.__class__)
+                    left, right = all[0], all[-1]
+                    self.left = left
+                    self.right = right
+                    left.right = self
+                    right.left = self
+        #
+        D = Directions
+        self.assertEqual(D.NORTH.value, (1, 0))
+        self.assertTrue(D.NORTH.left is D.WEST)
+        self.assertTrue(D.SOUTH.right is D.WEST)
+
+    def test_self_referential_rock_paper_scissors(self):
+        # https://stackoverflow.com/a/57085357/208880
+        class RPS(Enum):
+            #
+            Rock = "rock"
+            Paper = "paper"
+            Scissors = "scissors"
+            #
+            def __init__(self, value):
+                if len(self.__class__):
+                    # make links
+                    all = list(self.__class__)
+                    first, previous = all[0], all[-1]
+                    first.beats = self
+                    self.beats = previous
+        #
+        self.assertTrue(RPS.Rock.beats is RPS.Scissors)
+        self.assertTrue(RPS.Scissors.beats is RPS.Paper)
+        self.assertTrue(RPS.Paper.beats is RPS.Rock)
+
+    def test_arduino_headers(self):
+        # https://stackoverflow.com/q/65048495/208880
+        class CHeader(Enum):
+            def __init_subclass__(cls, **kwds):
+                # write Enums to C header file
+                cls_name = cls.__name__
+                header_path = getattr(cls, '_%s__header' % cls_name)
+                with open(header_path, 'w') as fh:
+                    fh.write('initial header stuff here\n')
+                    for enum in cls:
+                        fh.write('#define %s %r\n' % (enum.name, enum.value))
+        class Arduino(CHeader):
+            __header = os.path.join(tempdir, 'arduino.h')
+            ONE = 1
+            TWO = 2
+        with open(os.path.join(tempdir, 'arduino.h')) as fh:
+                data = fh.read()
+        self.assertEqual(textwrap.dedent("""\
+                initial header stuff here
+                #define ONE 1
+                #define TWO 2
+                """),
+                data,
+                )
+
+    def test_create_C_like_Enum(self):
+        # https://stackoverflow.com/a/35965438/208880
+        class Id(Enum, start=0):
+            #
+            NONE  # 0x0
+            HEARTBEAT  # 0x1
+            FLUID_TRANSFER_REQUEST
+            FLUID_TRANSFER_STATUS_MSG
+            FLUID_TRANSFER_ERROR_MSG
+            # ...
+            #
+            # Camera App Messages
+            START_SENDING_PICTURES = 0x010000
+            STOP_SENDING_PICTURES
+            START_RECORDING_VIDEO_REQ
+            STOP_RECORDING_VIDEO_REQ
+            # ...
+            #
+            # Sensor Calibration
+            VOLUME_REQUEST = 0x020000
+            START_CAL
+            CLI_COMMAND_REQUEST
+            CLI_COMMAND_RESPONSE
+            #
+            # File Mananger
+            NEW_DELIVERY_REQ = 0x30000
+            GET_DELIVERY_FILE_REQ
+            GET_FILE_REQ
+            #
+            ACK_NACK
+            RESPONSE
+            #
+            LAST_ID
+        #
+        self.assertEqual(Id.NONE.value, 0)
+        self.assertEqual(Id.FLUID_TRANSFER_ERROR_MSG.value, 4)
+        self.assertEqual(Id.START_SENDING_PICTURES.value, 0x010000)
+        self.assertEqual(Id.STOP_RECORDING_VIDEO_REQ.Value, 0x010003)
+        self.assertEqual(Id.START_CAL.value, 0x020001)
+        self.assertEqual(Id.LAST_ID.value, 0x30005)
+
 if __name__ == '__main__':
-    main()
+    tempdir = tempfile.mkdtemp()
+    try:
+        unittest.main()
+    finally:
+        shutil.rmtree(tempdir, True)
